"""
Videoanalytics sample

Command line to run:
python videoanalytics_sample.py -i camera.mp4 -m ssd300.xml -w ssd300.bin \ 
-c pascal_voc.txt -c_d 2 6 7 14 19 \
-l "C:\Program Files (x86)\IntelSWTools\openvino\deployment_tools\inference_engine\bin\intel64\Release\cpu_extension_avx2.dll" 
"""

from openvino.inference_engine import IENetwork, IECore
import os
import cv2
import sys
import argparse
import numpy as np
import logging as log
sys.path.append('../lib/')
from ie_detector import InferenceEngineDetector
from matching_tracker import MatchingTracker
from videoanalytics import Videoanalytics


def build_argparser():
    parser = argparse.ArgumentParser()
    parser.add_argument('-m', '--model', help='Path to an .xml \
        file with a trained model.', required=True, type=str)
    parser.add_argument('-w', '--weights', help='Path to an .bin file \
        with a trained weights.', required=True, type=str)
    parser.add_argument('-i', '--input', help='Path to \
        input video', required=True, type=str)
    parser.add_argument('-l', '--cpu_extension', help='MKLDNN \
        (CPU)-targeted custom layers. Absolute path to a shared library \
        with the kernels implementation', type=str, default=None)
    parser.add_argument('-d', '--device', help='Specify the target \
        device to infer on; CPU, GPU, FPGA or MYRIAD is acceptable. \
        Sample will look for a suitable plugin for device specified \
        (CPU by default)', default='CPU', type=str)
    parser.add_argument('-c', '--classes', help='File containing classes \
        names', type=str, default=None)
    parser.add_argument('-c_d', '--classes_detected', help='List of classe ids \
        to calc statistics', type=int, nargs='+', default='2 6 7 19')
    return parser

def main():
    log.basicConfig(format="[ %(levelname)s ] %(message)s",
                    level=log.INFO, stream=sys.stdout)
    args = build_argparser().parse_args()

    log.info("Start videoanalytics sample")

    ie_detector = InferenceEngineDetector(configPath=args.model,
        weightsPath=args.weights, device=args.device,
        extension=args.cpu_extension, classesPath = args.classes)

    tracker = MatchingTracker()

    videoanalytics = Videoanalytics(args.classes_detected, args.classes)

    cap = cv2.VideoCapture(args.input)

    timestamp = 0
    while(cap.isOpened()):
        timestamp += 1
        _, frame = cap.read()

        detections_mat = ie_detector.detect(frame)
        detections = tracker.filter_detections(detections_mat, 0.5)

        tracker.process_new_frame(frame, detections, timestamp)

        result_image = videoanalytics.draw_videoanalytics(frame, tracker)

        cv2.imshow('Videanalytics', result_image)
        if cv2.waitKey(1) & 0xFF == ord('q'):
            break

    cap.release()
    cv2.destroyAllWindows()
    return


if __name__ == '__main__':
    sys.exit(main())
