"""
Objects tracker using matching
"""

import cv2
import numpy as np
from openvino.inference_engine import IENetwork, IECore
from collections import namedtuple
from scipy.optimize import linear_sum_assignment
import math
import sys

DetectedObject = namedtuple('DetectedObject',
                            [
                                'confidence',
                                'frame_idx',
                                'object_id',
                                'timestamp',
                                'class_id',
                                'x_left',
                                'y_bottom',
                                'x_right',
                                'y_top',
                            ])


class Tracklet():
    """
    Class that stores sequence of DetectedObject objects
    """

    def __init__(self, detection=[]):
        self._trackedObjects = []
        self._trackedObjects.append(detection)

    def __len__(self):
        return len(self._trackedObjects)

    def __getitem__(self, position):
        return self._trackedObjects[position]

    def add_new_detection(self, detection):
        self._trackedObjects.append(detection)

    def __str__(self):
        return 'Track: {} elements'.format(len(self))


class MatchingTracker:
    """
    Class that stores and processes tracks based on images
    """

    def __init__(self, dist_weight=0.02, shape_affinity_weight=0.5,
            place_affinity_weight=0.7):
        self._tracks = []
        self._tracks_counter = 0
        self._dist_weight = dist_weight
        self._shape_affinity_weight = shape_affinity_weight
        self._place_affinity_weight = place_affinity_weight
        return

    def add_new_track(self, detection):
        track = Tracklet(detection)
        self._tracks.append(track)

    def filter_detections(self, detect_mat, threshold=0.5):
        detect_mat = detect_mat[0, 0, :, :]
        detections = []
        for i in range(detect_mat.shape[0]):
            # Parse one string in ie_detection_output
            conf = detect_mat[i, 2]
            if conf > threshold:
                detection = DetectedObject(
                    detect_mat[i, 2],
                    -1,
                    -1,
                    -1,
                    detect_mat[i, 1],
                    detect_mat[i, 3],
                    detect_mat[i, 4],
                    detect_mat[i, 5],
                    detect_mat[i, 6])
                detections.append(detection)
        return detections

    def process_new_frame(self, frame, detections, timestamp):
        """
        First step is to compute an affinity matrix between tracklets and detections.
        The affinity equals to place_affinity * shape_affinity.
        Where appearance is 1 - distance(tracklet_fast_dscr, detection_fast_dscr).
        Second step is to solve the assignment problem using Kuhn-Munkres algorithm.
        If correspondence between some tracklet and detection is established with
        low confidence (affinity) then the strong descriptor is used to determine
        if there is correspondence between tracklet and detection.
        """
        if self._tracks and detections:
            row_indexes, col_indexes = self._solve_assignment_problem(
                self._tracks, detections)
            # For each assignment
            for i in range(len(row_indexes)):
                row_id = row_indexes[i]
                col_id = col_indexes[i]
                # If we find existing track and existing detection
                if col_id < len(self._tracks) and col_id < len(detections):
                    # Add detection to track if objects are close
                    dist = 1.0 - \
                        self._affinity(
                            self._tracks[row_id][-1], detections[col_id])
                    if dist > self._dist_weight:
                        self._tracks[row_id].add_new_detection(
                            detections[col_id])
                elif col_id < len(detections):
                    self.add_new_track(detections[col_id])
        else:
            # Add new tracks
            for id, detection in enumerate(detections):
                self.add_new_track(detection)
        return

    def _solve_assignment_problem(self, tracks, detections):
        dissimilarity_mat = self._compute_dissimilarity_matrix(
            tracks, detections)
        row_ind, col_ind = linear_sum_assignment(dissimilarity_mat)
        return row_ind, col_ind

    def _compute_dissimilarity_matrix(self, tracks, detections):
        size = max(len(tracks), len(detections))
        diss_mat = np.zeros(shape=(size, size), dtype=float)
        for i in range(len(tracks)):
            for j in range(len(detections)):
                diss_mat[i, j] = 1.0 - \
                    self._affinity(tracks[i][-1], detections[j])
        return diss_mat

    def _affinity(self, obj1, obj2):
        """
        Calculate affinity between two objects
        """
        shp_aff = self._shape_affinity(obj1, obj2, self._shape_affinity_weight)
        mot_aff = self._place_affinity(obj1, obj2, self._place_affinity_weight)
        return shp_aff * mot_aff

    def _shape_affinity(self, obj1, obj2, weight):
        obj1_width = obj1.x_right-obj1.x_left
        obj2_width = obj2.x_right-obj2.x_left
        obj1_height = obj1.y_top - obj1.y_bottom
        obj2_height = obj2.y_top - obj2.y_bottom
        w_dist = abs(obj1_width - obj2_width) / (obj1_width + obj2_width)
        h_dist = abs(obj1_height - obj2_height) / (obj1_height + obj2_height)
        return math.exp(-weight * (w_dist + h_dist))

    def _place_affinity(self, o1, o2, weight):
        obj1_x = (o1.x_left + o1.x_right) * 0.5
        obj1_y = (o1.y_top + o1.y_bottom) * 0.5
        obj2_x = (o2.x_left + o2.x_right) * 0.5
        obj2_y = (o2.y_top + o2.y_bottom) * 0.5
        obj2_width = o2.x_right-o2.x_left
        obj2_height = o2.y_top - o2.y_bottom
        x_dist = ((obj1_x - obj2_x) ** 2) / (obj2_width ** 2)
        y_dist = ((obj1_y - obj2_y) ** 2) / (obj2_height ** 2)
        return math.exp(-weight * (x_dist + y_dist))

    def draw_active_tracks(self, image):
        w, h = image.shape[:2]
        for i, track in enumerate(self._tracks):
            # Draw one track from segments
            for i in range(len(track)-1):
                cv2.line(img=image,
                    pt1=(int((track[i].x_left + track[i].x_right) * h) // 2,
                         int((track[i].y_bottom + track[i].y_top) * w) // 2),
                    pt2=(int((track[i+1].x_left + track[i+1].x_right) * h) // 2,
                         int((track[i+1].y_bottom + track[i+1].y_top) * w) // 2),
                    color=(0, 255, 0), thickness=3)
        return image
